	SUBROUTINE FORCES
************************************************************
*   Calculate derivative of the total energy over nuclear  *
*   displacements. The Pil`au forces are assumed to be      *
*   taken in "rigid" approximation.                        *
*   Written by S.Yu.Savrasov                               *
************************************************************
	IMPLICIT REAL*8 (A-H,O-Z)
	INCLUDE 'PARAM.DAT'
	INTEGER ILST(NSORTMAX),ILSB(NSORTMAX),ILSV(NSORTMAX),IS(NPAT)
	COMPLEX*16 FBND(3,NPAT),FMLT(3,NPAT),FCOU(3,NPAT),FEXC(3,NPAT)
     &    ,FMT0(3,NPAT),FTOT(3,NPAT)
	DIMENSION  TAU(3,NPAT)
	CHARACTER*10 TXTEL(NSORTMAX)
	COMMON /CONTROL/MAN,LMTO,NLR,NINT,NPF               !!! from init
	COMMON /PARAM/  NSORT,IS,ILST,ILSB,ILSV,NATOM       !!! from init
	COMMON /BASIS/  TAU                                 !!! from init
	COMMON /SPIN/   NSPIN                               !!! from init
	COMMON /TXTEL/  TXTEL                               !!! from init
	COMMON /UNIT/   IUN                                 !!! from init
	IF(NPF.NE.1)RETURN
C   CALCULATE DIFFERENT FORCES
	CALL BNDFORCE(FBND)          !!! band contribution to the Pilau force
	CALL MLTFORCE(FMLT)          !!! multipole contribution to the el.force
	CALL SRFFORCE(FCOU,FEXC,FMT0)!!! integrals with Ro and Vc,Exc,Vmt
C   PRINT DIFFERENT FORCES
	DO 20 IATOM=1,NATOM
	ISORT=IS(IATOM)
	DO 10 I=1,3
	FCOU(I,IATOM)=FCOU(I,IATOM)+FMLT(I,IATOM)
	FTOT(I,IATOM)=FBND(I,IATOM)+FCOU(I,IATOM)+FEXC(I,IATOM)+
     &  FMT0(I,IATOM)
  10    CONTINUE
  20	CONTINUE
        WRITE(IUN,1010)
        DO 30 IATOM=1,NATOM
        WRITE(IUN,1020)(-DREAL(FTOT(I,IATOM)),I=1,3),TXTEL(IS(IATOM))
  30    CONTINUE
1010   FORMAT(1X,'* CALCULATED FORCES AT THE CENTERS OF ATOMS ------>')
1020   FORMAT(1X,'* TOTAL FORCE: Fx=',G14.7,'; Fy=',G14.7,'; Fz=',G14.7,
     &  1X,A10)
        WRITE(IUN,1050)
        WRITE(IUN,1060)
	DO 40 IATOM=1,NATOM
	WRITE(IUN,1070)(TAU(I,IATOM),I=1,3),TXTEL(ISORT)
	WRITE(IUN,1110)(DREAL(FBND(I,IATOM)),I=1,3)
	WRITE(IUN,1120)(DREAL(FCOU(I,IATOM)),I=1,3)
	WRITE(IUN,1130)(DREAL(FEXC(I,IATOM)),I=1,3)
	WRITE(IUN,1140)(DREAL(FMT0(I,IATOM)),I=1,3)
	WRITE(IUN,1150)(DREAL(FTOT(I,IATOM)),I=1,3)
  40    CONTINUE
	WRITE(IUN,1050)
1050   FORMAT(1X,'***************************************************')
1060   FORMAT(1X,'* DIFFERENT CONTRIBUTIONS TO DERIVATIVES    ------>')
1070   FORMAT(1X,'* Position --> (',G12.5,',',G12.5,',',G12.5,') ',A10)
1110   FORMAT(1X,'* BAND  DERIV: Fx=',G14.7,'; Fy=',G14.7,'; Fz=',G14.7)
1120   FORMAT(1X,'* COUL. DERIV: Fx=',G14.7,'; Fy=',G14.7,'; Fz=',G14.7)
1130   FORMAT(1X,'* EXCH. DERIV: Fx=',G14.7,'; Fy=',G14.7,'; Fz=',G14.7)
1140   FORMAT(1X,'* MT-Z. DERIV: Fx=',G14.7,'; Fy=',G14.7,'; Fz=',G14.7)
1150   FORMAT(1X,'* TOTAL DERIV: Fx=',G14.7,'; Fy=',G14.7,'; Fz=',G14.7)
	END




	SUBROUTINE BNDFORCE(FBND)
**************************************************************
*   Calculate band contribution to the Pilau force assuming  *
*   change in the strucure constants only.                   *
*   Written by S.Yu.Savrasov.                                *
**************************************************************
	IMPLICIT REAL*8 (A-H,O-Z)
	INCLUDE 'PARAM.DAT'
	INTEGER  ILST(NSORTMAX),ILSB(NSORTMAX),ILSV(NSORTMAX),IS(NPAT)
	COMPLEX*16 PFVAL(-1:1,NPAT,NPANMAX,NSPINMAX),FBND(3,NPAT)
     &    ,FSUM(-1:1)
	DIMENSION  TAU(3,NPAT),TOS(NPANMAX,0:1,NSPINMAX)
	CHARACTER  TXTEL(NSORTMAX)*10
	COMMON /PARAM/  NSORT,IS,ILST,ILSB,ILSV,NATOM       !!! from init
	COMMON /BASIS/  TAU                                 !!! from init
	COMMON /SPIN/   NSPIN                               !!! from init
	COMMON /KAPPAR/ NPAN                                !!! from init
	COMMON /EFERMI/ EF                                  !!! from sefermi
	COMMON /PILAUF/ PFVAL                               !!! from ttrint3
	COMMON /UNIT/   IUN                                 !!! from init
	COMMON /TXTEL/  TXTEL                               !!! from init
	DATA PI/3.141592653589793D0/
C  TRANSFERE TO CARTESIAN SYSTEM
	DO 30 IATOM=1,NATOM
	DO 20 MY=-1,1
	FSUM(MY)=(0.D0,0.D0)
	DO 10 ISPIN=1,NSPIN
	DO 10 IPAN=1,NPAN
	FSUM(MY)=FSUM(MY)+PFVAL(MY,IATOM,IPAN,NSPIN)
  10    CONTINUE
  20    CONTINUE
	CALL TRANSFER(FSUM,FBND(1,IATOM))
  30    CONTINUE
	END

	SUBROUTINE MLTFORCE(FMLT)
************************************************************
*   Calculate multipole contribution to the electrostatic  *
*   force. Written by S.Yu.Savrasov                        *
************************************************************
	IMPLICIT REAL*8 (A-H,O-Z)
	INCLUDE 'PARAM.DAT'
	INTEGER ILST(NSORTMAX),ILSB(NSORTMAX),ILSV(NSORTMAX),IS(NPAT)
     &   ,LMSYM(LMMAXV,NPAT),ICENV(0:NENVMAX,NPAT),NCEL(NPAT)
	COMPLEX*16 QASA(NSYMMAX,NPAT),QINT(NSYMMAX,NPAT)
     &   ,AMTOT(NSYMMAX,NPAT,NSPINMAX),SS(LMMAXV1,LMMAXV1)
     &   ,FORCE(-1:1,NPAT),FRC(-1:1,NPAT),FMLT(3,NPAT),ZTOT
     &   ,TCH(NSYMMAX,NPAT,2,NSPINMAX),SR(LMMAXV1,LMMAXV1)
     &   ,TRH(LMMAXV1,0:NENVMAX,NPAT,NSPINMAX),AC(LMMAXV1)
	DIMENSION SMT(NSORTMAX),TAU(3,NPAT),Z(NSORTMAX),ZCOR(NSORTMAX)
     &   ,C(0:LMAXV+1,LMMAXV1,LMMAXV1),CENV(3,0:NENVMAX,NPAT)
	LOGICAL SYM(LMMAXV,NPAT)
	CHARACTER TXTEL(NSORTMAX)*10
	COMMON /CONTROL/LIFT,LMTO,NCOU,NINT,NPF             !!! from init
	COMMON /PARAM/  NSORT,IS,ILST,ILSB,ILSV,NATOM       !!! from init
	COMMON /POTCOM/ SWZ,SMT                             !!! from init
	COMMON /BASIS/  TAU                                 !!! from init
	COMMON /SYMMLM/ SYM,LMSYM                           !!! from symmlm
	COMMON /MULTRO/ Z,ZCOR,TCH,AMTOT,QASA,QINT,TRH      !!! from mult
	COMMON /CELMESH/NCEL,CENV,ICENV                     !!! from srfmsh
	COMMON /SPIN/   NSPIN                               !!! from init
	COMMON /GAUNT/  C                                   !!! from gaunt
	COMMON /UNIT/   IUN                                 !!! from init
	COMMON /TXTEL/  TXTEL                               !!! from init
	DATA PI/3.141592653589793D0/
	SQPI43=SQRT(4.D0*PI/3.D0)
	DO 20 INUC=1,NATOM
	DO 10 MY=-1,1
	FRC(MY,INUC)=(0.D0,0.D0)
   10   CONTINUE
   20   CONTINUE
C ***** DO GLOBALLY OVER ALL ATOMS IN CELL *****
	DO 4000 IATOM=1,NATOM              !!! over atoms
	ISORT=IS(IATOM)
	DO 3000 IATOM1=1,NATOM             !!! over atoms
	ISORT1=IS(IATOM1)
	CALL STRMAT(IATOM,IATOM1,SS,SR)
	DO 60 L=0,ILSV(ISORT)+1
	DO 50 M=-L,L
	LM=L*(L+1)+M+1
	AC(LM)=(0.D0,0.D0)
	DO 25 IENV=1,NCEL(IATOM)
	IF(ICENV(IENV,IATOM).NE.IATOM1)GOTO25
	IF(NCOU.EQ.0)GOTO25
	AC(LM)=AC(LM)-(TRH(LM,IENV,IATOM,1)+TRH(LM,IENV,IATOM,NSPIN))/
     &  2.D0*DBLE(NSPIN)
   25   CONTINUE
	DO 40 L1=0,ILSV(ISORT1)
	SL1=1.D0/SMT(ISORT1)/DBLE(2*L1+1)
	DO 30 M1=-L1,L1
	L1M1=L1*(L1+1)+M1+1
	IF(SYM(L1M1,IATOM1))THEN
	ISYM1=LMSYM(L1M1,IATOM1)
	AC(LM)=AC(LM)+SS(LM,L1M1)*
     &  (AMTOT(ISYM1,IATOM1,1)-TCH(ISYM1,IATOM1,2,1)+
     &  AMTOT(ISYM1,IATOM1,NSPIN)-TCH(ISYM1,IATOM1,2,NSPIN))/
     &  2.D0*DFLOAT(NSPIN)*SL1
	AC(LM)=AC(LM)+SR(LM,L1M1)*
     &  (TCH(ISYM1,IATOM1,2,1)+TCH(ISYM1,IATOM1,2,NSPIN))/
     &  2.D0*DFLOAT(NSPIN)*SL1
	ENDIF
  30    CONTINUE                           !!! over m1
  40    CONTINUE                           !!! over l1
  50    CONTINUE                           !!! over m
  60    CONTINUE                           !!! over l
	DO 2000 INUC=1,NATOM               !!! over displacements
	DSYM=0.D0
	IF(IATOM .EQ.INUC)DSYM=DSYM+1.D0
	IF(IATOM1.EQ.INUC)DSYM=DSYM-1.D0
	DO 1000 MY=-1,1                    !!! over polarizations
C CALCULATE MULTIPOLE CONTRIBUTION TO THE ELECTRSTATIC FORCE
	DO 80 L=0,ILSV(ISORT)              !!! over l
	SL=1.D0/SMT(ISORT)/DBLE(2*L+1)     !!! over m
	DO 70 M=-L,L
	LM=L*(L+1)+M+1
	LP1MY=(L+1)*(L+2)+M-MY+1
	IF(SYM(LM,IATOM))THEN
	ISYM=LMSYM(LM,IATOM)
	FRC(MY,INUC)=FRC(MY,INUC)-SL*(0.D0,1.D0)*SQPI43*
     &  DBLE(2*L+1)/SMT(ISORT)*CONJG(AMTOT(ISYM,IATOM,1)+
     &  AMTOT(ISYM,IATOM,NSPIN))/2.D0*DFLOAT(NSPIN)*
     &  C(0,LM,LP1MY)*AC(LP1MY)*DSYM
	ENDIF
  70    CONTINUE                           !!! over m
  80    CONTINUE                           !!! over l
1000    CONTINUE                           !!! over polarizations
2000    CONTINUE                           !!! over displacements
3000    CONTINUE                           !!! over atoms
4000    CONTINUE                           !!! over atoms
C TRANSFERE TO CARTESIAN SYSTEM
	DO 5000 INUC=1,NATOM
	CALL TRANSFER(FRC(-1,INUC),FMLT(1,INUC))
5000    CONTINUE
	END




	SUBROUTINE SRFFORCE(FCOU,FEXC,FMT0)
************************************************************
*  Calculate integrals over atomic cells with the charge   *
*  density and different contributions to the potential.   *
*  Written by S.Yu.Savrasov                                *
************************************************************
	IMPLICIT REAL*8 (A-H,O-Z)
	INCLUDE 'PARAM.DAT'
	INTEGER ILST(NSORTMAX),ILSB(NSORTMAX),ILSV(NSORTMAX)
     &    ,IS(NPAT),NHSORT(NSORTMAX),NPOL(0:NSORTMAX)
     &    ,NSHE(NSORTMAX),LMSYM(LMMAXV,NPAT)
	COMPLEX*16 DIM,FCOU(3,NPAT),FEXC(3,NPAT),FMT0(3,NPAT)
     &    ,VINT(0:NSHEMAX,NSYMMAX,NPAT,NSPINMAX)
     &    ,XCINT(0:NSHEMAX,NSYMMAX,NPAT,NSPINMAX)
     &    ,RO   (0:NRADMAX,NSYMMAX,NPAT,NSPINMAX)
     &    ,ROCOR(0:NRADMAX,NSYMMAX,NPAT,NSPINMAX)
     &    ,ROINT(0:NSHEMAX,NSYMMAX,NPAT,NSPINMAX)
     &    ,ROVALDS(3,NSYMMAX,NPAT,NSPINMAX)
	DIMENSION SMT(NSORTMAX),SROU(NSORTMAX),PCEL(3,NMSHMAX)
     &    ,RHOINT(0:NSHEMAX,NMSHMAX,NSPINMAX)
     &    ,POTINT(0:NSHEMAX,NMSHMAX,NSPINMAX)
     &    ,EXCINT(0:NSHEMAX,NMSHMAX,NSPINMAX)
     &    ,D0CEL(0:NSHEMAX,NMSHMAX),DWCEL(0:NSHEMAX,NMSHMAX)
	CHARACTER TXTEL(NSORTMAX)*10
	LOGICAL SYM(LMMAXV,NPAT)
	COMMON /PARAM/  NSORT,IS,ILST,ILSB,ILSV,NATOM,NHSORT!!! from init
	COMMON /POTCOM/ SWZ,SMT                             !!! from vfull
	COMMON /DENS/   RO,ROVALDS,ROCOR,ROINT              !!! from roint
	COMMON /VINT/   VINT                                !!! from vint
	COMMON /STORE2/ XCINT,RHOINT,POTINT,EXCINT,DWCEL,D0CEL
	COMMON /SYMMLM/ SYM,LMSYM                           !!! from lmsym
	COMMON /INTMESH/NSHE,NPOL,SROU                      !!! from init
	COMMON /VZERO/  VMT0                                !!! from vzero
	COMMON /SPIN/   NSPIN                               !!! from init
	COMMON /UNIT/   IUN                                 !!! from init
	DATA PI/3.141592653589793D0/,DIM/(0.D0,1.D0)/
	SQPI4=SQRT(4.D0*PI)
C  SET UP FULL POTENTIAL TO ZERO
	DO 50 IATOM=1,NATOM               !!! over atoms
	ISORT=IS(IATOM)
	DO 40 ISPIN=1,NSPIN               !!! over spins
	DO 30 L=0,ILSV(ISORT)             !!! over l
	DO 20 M=-L,L                      !!! over m
	LM=L*(L+1)+M+1
	IF(SYM(LM,IATOM))THEN             !!! for non-zero components
	ISYM=LMSYM(LM,IATOM)
	DO 10 IRAD=0,NSHE(ISORT)+1        !!! over int.points
	VINT(IRAD,ISYM,IATOM,ISPIN)=(0.D0,0.D0)
  10    CONTINUE                          !!! over int.points
	ENDIF                             !!! for non-zero components
  20    CONTINUE                          !!! over m
  30    CONTINUE                          !!! over l
  40    CONTINUE                          !!! over spins
  50    CONTINUE                          !!! over atoms
C  CALCULATE EXCHANGE-CORRELATION ENERGY
	CALL VXINT(2)  ! calculate exchange-correlation energy in the int.reg.
C  SET UP FULL POTENTIAL TO ZERO
	DO 95 IATOM=1,NATOM               !!! over atoms
	ISORT=IS(IATOM)
	DO 90 ISPIN=1,NSPIN               !!! over spins
	DO 80 L=0,ILSV(ISORT)             !!! over l
	DO 70 M=-L,L                      !!! over m
	LM=L*(L+1)+M+1
	IF(SYM(LM,IATOM))THEN             !!! for non-zero components
	ISYM=LMSYM(LM,IATOM)
	DO 60 IRAD=0,NSHE(ISORT)+1        !!! over int.points
	XCINT(IRAD,ISYM,IATOM,ISPIN)=VINT(IRAD,ISYM,IATOM,ISPIN)
	VINT(IRAD,ISYM,IATOM,ISPIN)=(0.D0,0.D0)
  60    CONTINUE                          !!! over int.points
	ENDIF                             !!! for non-zero components
  70    CONTINUE                          !!! over m
  80    CONTINUE                          !!! over l
  90    CONTINUE                          !!! over spins
  95    CONTINUE                          !!! over atoms
	CALL VCINT  
C  SET UP FORCE TO ZERO
	DO 100 INUC=1,NATOM                !!! over displacements
	DO 100 IA=1,3                      !!! over polarizations
	FCOU(IA,INUC)=(0.D0,0.D0)
	FEXC(IA,INUC)=(0.D0,0.D0)
	FMT0(IA,INUC)=(0.D0,0.D0)
 100    CONTINUE                           !!! over inuc,ia
C ****  DO GLOBALLY OVER ATOMS  ****
	DO 150 IATOM=1,NATOM               !!! over atoms
	ISORT=IS(IATOM)
C ****  DO OVER POLARIZATINS AND DISPLACEMENTS ****
	DO 145 IA=1,3                       !!! over polarizations
	CALL DELSGRD(IATOM,0,IA,NMSH,D0CEL,PCEL)
	IF(IA.EQ.1)THEN
	CALL SFUNIN(IATOM,NMSH,PCEL,ROINT,RHOINT) ! charge density
	CALL SFUNIN(IATOM,NMSH,PCEL,V INT,POTINT) ! coul.potential
	CALL SFUNIN(IATOM,NMSH,PCEL,XCINT,EXCINT) ! exc.corr.energy 
	ENDIF
	DO 140 INUC=1,NATOM                 !!! over displacements
	CALL DELSGRD(IATOM,INUC,IA,NMSH,DWCEL,PCEL)
	DO 130 ISPIN=1,NSPIN                !!! over spins
	DO 120 IMSH=1,NMSH                  !!! over surface grid
	DO 110 IRAD=0,NSHE(ISORT)+1         !!! over radial points
	PWGT=DWCEL(IRAD,IMSH)
	IF(IATOM.EQ.INUC)PWGT=PWGT-D0CEL(IRAD,IMSH)
	FCOU(IA,INUC)=FCOU(IA,INUC)+RHOINT(IRAD,IMSH,ISPIN)*
     &  POTINT(IRAD,IMSH,ISPIN)*PWGT
	FEXC(IA,INUC)=FEXC(IA,INUC)+RHOINT(IRAD,IMSH,ISPIN)*
     &  EXCINT(IRAD,IMSH,ISPIN)*PWGT
	FMT0(IA,INUC)=FMT0(IA,INUC)-RHOINT(IRAD,IMSH,ISPIN)*
     &  VMT0*PWGT
 110    CONTINUE                            !!! over radial points
 120    CONTINUE                            !!! over surface grid
 130    CONTINUE                            !!! over spins
 140    CONTINUE                            !!! over displacememts
 145    CONTINUE                            !!! over polarizations
 150    CONTINUE                            !!! over atoms
	END





	SUBROUTINE TRANSFER(A,B)
************************************************************
*  Transfere from cyclic to cartesian system.              *
************************************************************
	IMPLICIT REAL*8 (A-H,O-Z)
	COMPLEX*16 TR(-1:1,3),A(-1:1),B(3)
	DATA TR/( 0.7071068,0.0000000),(0.0000000, 0.0000000)
     &       ,(-0.7071068,0.0000000),(0.0000000,+0.7071068)
     &       ,( 0.0000000,0.0000000),(0.0000000,+0.7071068)
     &       ,( 0.0000000,0.0000000),(1.0000000, 0.0000000)
     &       ,( 0.0000000,0.0000000)/
	DO 20 I=1,3
	B(I)=(0.D0,0.D0)
	DO 10 MY=-1,1
	B(I)=B(I)+A(MY)*TR(MY,I)
  10    CONTINUE
  20    CONTINUE        
	END
