	SUBROUTINE MCTTRINT0(E0,EF,OT,AT,AD,TS,DS,TBLO,TRET)
************************************************************
*  Finds weights for integrating over microtetrahedron:    *
*                                                          *
*        /  3                                              *
*        \ d k  R(k)*theta(Ef-E(k))  - TOS like            *
*        /                                                 *
*       ttr                                                *
*                                                          *
*  or                                                      *
*                                                          *
*        /  3                                              *
*        \ d k  R(k)*delta(Ef-E(k))  - DOS like            *
*        /                                                 *
*       ttr                                                *
*                                                          *
*  where R(k) are 1,kx,ky,kz (for the matrix elements      *
*  evaluation). The linear interpolation is supposed.      *
*  Inputs:                                                 *
*   E0(I)  - energies E(k) at the corners.                 *
*   EF     - Fermi energy.                                 *
*   OT     - Volume of tetrahedron.                        *
*  Keys:                                                   *
*   Tblo=T - standard tetrahedron method.                  *
*   Tblo=B - with Bloechl corrections.                     *
*   Tret=W - returns weights for the corners  (i=1...4).   *
*   Tret=I - returns integrals for 1,kx,ky,kz (i=1...4).   *
*  Outputs:                                                *
*   AT(I)  - TOS weights/integrals for Tret=W/I.           *
*   AD(I)  - DOS weights/integrals for Tret=W/I.           *
*   TS     - total number of states.                       *
*   DS     - total density of states.                      *
*  Comments:                                               *
*   Option Tret=I is not curreantly avaiable.              *
************************************************************
	IMPLICIT REAL*8 (A-H,O-Z)
	DIMENSION ET(4),E0(4),IND(4),L(4),AT(4),AD(4)
     &    ,BT(4),BD(4)
	CHARACTER TBLO*1,TRET*1
	DO I=1,4
        IND(I)=1
	ENDDO
C ORDER E0
	DO I=1,3
	I1=I+1
	DO J=I1,4
	IF(E0(I).GT.E0(J))IND(J)=IND(J)+1
	IF(E0(I).LE.E0(J))IND(I)=IND(I)+1
	ENDDO
	ENDDO
	DO I=1,4
	JJ=IND(I)
	L(JJ)=I
        ET(JJ)=E0(I)
	ENDDO
C FIND CASE 
	IF(EF.LE.ET(4))THEN
C Tetrahedron is empty
	TS=0.D0
	DS=0.D0
	DDS=0.D0
	DO I=1,4
	AT(I)=0.D0
	AD(I)=0.D0
	ENDDO
        ELSEIF(EF.GT.ET(1))THEN
C Tetrahedron is fully occupied
	TS=OT
	DS=0.D0
	DDS=0.D0
	DO I=1,4
	AD(I)=0.D0
	AT(I)=OT/4.D0
	ENDDO
        ELSEIF(EF.LE.ET(3))THEN
C Tetrahedron is partially occupied
	EF4=EF-ET(4)
	E24=EF4/(ET(2)-ET(4))
	E14=EF4/(ET(1)-ET(4))
	E34=EF4/(ET(3)-ET(4))
	TMP=E24*E14*E34*OT
	DS=TMP*3.D0/EF4
	DDS=DS*2.D0/EF4       !!! dD(Ef)/dEf for Bloechl correction
	TS=TMP
	AD(L(1))=DS*E14/3.D0
	AD(L(2))=DS*E24/3.D0
	AD(L(3))=DS*E34/3.D0
	AD(L(4))=DS*(1.D0-(E14+E24+E34)/3.D0)
	AT(L(1))=TS*E14/4.D0
	AT(L(2))=TS*E24/4.D0
	AT(L(3))=TS*E34/4.D0
	AT(L(4))=TS*(1.D0-(E14+E24+E34)/4.D0)
	ELSEIF(EF.GT.ET(3).AND.EF.LE.ET(2))THEN
C Tetrahedron is partially occupied
C SECTION IS A QUADRANGLE
	OM23=ET(2)-ET(3)
C DIVIDING INTO TWO TETRAHEDRA
C THE FIRST TETRAHEDRON
	DOM1=EF-ET(4)
	DOM11=ET(1)-ET(4)
	DOM12=ET(2)-ET(4)
	DOM13=DOM1
	F1=1.D0/DOM11/DOM12*(ET(2)-EF)/OM23
C THE SECOND TETRAHEDRON
	DOM2=ET(1)-EF
	DOM21=DOM2
	DOM22=ET(1)-ET(3)
	DOM23=ET(1)-ET(4)
	F2=1.D0/DOM22/DOM23
	G2=(EF-ET(3))/OM23
	DS=(DOM1*F1+DOM2*F2*G2)*3.D0*OT
	DDS=(DOM2/DOM22-DOM1/DOM12)/DOM11/OM23*6.D0*OT
	TS=(DOM1**2*F1+(1.D0-DOM2**2*F2)*G2)*OT
	DF1=3.D0*OT*DOM1*F1
	TF1=OT*DOM1*F1*DOM1
	DF2=3.D0*OT*DOM2*F2*G2
	TF2=OT*DOM2*F2*G2*DOM2
	DFF=(EF-ET(3))/(ET(2)-ET(3))
	AD(L(1))=DF2*(1.D0-DOM2/3.D0*(1.D0/DOM21+1.D0/DOM22+
     &  1.D0/DOM23))+DF1*DOM1/DOM11/3.D0
	AD(L(2))=DF1*DOM1/3.D0*(1.D0/DOM12+1.D0/DOM13*DFF)+
     &  DF2*DOM2/DOM21/3.D0*DFF
	AD(L(3))=DF2*DOM2/3.D0*(1.D0/DOM21+1.D0/DOM22-
     &  1.D0/DOM21*DFF)+DF1*DOM1/DOM13/3.D0*(1.D0-DFF)
	AD(L(4))=DF1*(1.D0-DOM1/3.D0*(1.D0/DOM11+1.D0/DOM12+
     &  1.D0/DOM13))+DF2*DOM2/DOM23/3.D0
	AT(L(1))=-TF2*(1.D0-DOM2/4.D0*(1.D0/DOM21+1.D0/DOM22+
     &  1.D0/DOM23))+TF1*DOM1/DOM11/4.D0+OT*G2/4.D0
	AT(L(2))=TF1*DOM1/4.D0*(1.D0/DOM12+1.D0/DOM13*DFF)-
     &  TF2*DOM2/DOM21/4.D0*DFF+OT*G2/4.D0*DFF
	AT(L(3))=-TF2*DOM2/4.D0*(1.D0/DOM21+1.D0/DOM22-
     &  1.D0/DOM21*DFF)+TF1*DOM1/DOM13/4.D0*(1.D0-DFF)+
     &  OT*G2/4.D0*(2.D0-DFF)
	AT(L(4))=TF1*(1.D0-DOM1/4.D0*(1.D0/DOM11+1.D0/DOM12+
     &  1.D0/DOM13))-TF2*DOM2/DOM23/4.D0+OT*G2/4.D0
	ELSEIF(EF.GT.ET(2))THEN
C Tetrahedron is partially occupied
	EF1=(EF-ET(1))
	E14=EF1/(ET(1)-ET(4))
	E12=EF1/(ET(1)-ET(2))
	E13=EF1/(ET(1)-ET(3))
	TMP=E12*E13*E14*OT
C   !NB TMP<0,EF1<0
	DS=TMP*3.D0/EF1
	DDS=DS*2.D0/EF1
	TS=OT+TMP
	AD(L(1))=DS*(1.D0+(E12+E13+E14)/3.D0)
	AD(L(2))=-DS*E12/3.D0
	AD(L(3))=-DS*E13/3.D0
	AD(L(4))=-DS*E14/3.D0
	AT(L(1))=OT/4.D0+TMP*(1.D0+(E12+E13+E14)/4.D0)
	AT(L(2))=OT/4.D0-TMP*E12/4.D0
	AT(L(3))=OT/4.D0-TMP*E13/4.D0
	AT(L(4))=OT/4.D0-TMP*E14/4.D0
	ENDIF
	IF(TBLO.EQ.'T')THEN
C DO NOT ADD BLOECHL CORRECTION
	ELSEIF(TBLO.EQ.'B')THEN
C ADD BLOECHL CORRECTION
	DO I=1,4
	DO J=1,4
	AT(L(I))=AT(L(I))+(ET(J)-ET(I))*DS*0.025D0
	AD(L(I))=AD(L(I))+(ET(J)-ET(I))*DDS*0.025D0
	ENDDO
	ENDDO
	ENDIF
C SET WEIGHTS/INTEGRALS ACCORDING TO TRET
	IF(TRET.EQ.'W')THEN
C WEIGHTS ARE READY
	CONTINUE
	ELSEIF(TRET.EQ.'I')THEN
  	STOP'INTEGRALS NOT AVAIABLE'
	ENDIF
	RETURN
	END




	SUBROUTINE MCTTRINT1(VINP,VT,A)
*****************************************************************
*    The calculation of the integrals over tetrahedron          *
*                                                               *
*        /  3     R(k)                                          *
*    A=  \ d k   ------                                         *
*        /        V(k)                                          *
*       ttr                                                     *
*                                                               *
*    (For instance V(k)=E(k)-E(k+q)+omega in the linear         *
*                                             response theory.) *
*    where V(k) is given at the vertexes of tetrahedron,        *
*    R(k) is 1,kx,ky,kz (for the matrix elements evaluation)    *
*    The linear interpolation is supposed.                      *
*    Written by S.Yu.Savrasov (Lebedev Physical Institute)      *
*****************************************************************
	IMPLICIT REAL*16 (A-H,O-Z)
	REAL*8 VINP(4),VT,A(4)
	DIMENSION V(4),D(4),ALN(4),ICON(6),LL(6),MM(6)
     &   ,T(2:4),KK(2),KP(5),NN2(2:4,6),NN3(2:4,3)
     &   ,NN4(2:4,5)
	DATA LL/1,1,1,2,2,3/,MM/2,3,4,3,4,4/,KP/4,3,2,0,1/
     &      ,NN2/4,3,2,3,4,2,3,2,4,4,4,2,4,2,4,2,4,4/
     &      ,NN3/2,3,3,3,2,3,3,3,2/
     &      ,NN4/3,3,2,3,2,3,2,3,3,3,3,3,3,3,3/
C   INITIALIZATION
	DO 1 I=1,4 
	V(I)=VINP(I)
	IF(ABS(V(I)).LE.1.D-08)V(I)=1.D-08
   1	CONTINUE
C   DEFINE CASE OF COINCIDING
	DO 20 I=1,4
	DO 10 J=I+1,4
	IF(ABS(V(I)-V(J)).LE.1.D-05)V(J)=V(I)
  10    CONTINUE
  20    CONTINUE
	ICASE=0
	K=0
	DO 40 I=1,4
	DO 30 J=I+1,4
	K=K+1
	ICON(K)=0
	IF(ABS(V(I)-V(J)).LE.1.D-05)ICON(K)=1
	ICASE=ICASE+ICON(K)
  30    CONTINUE
  40    CONTINUE
C
	IF(ICASE.EQ.0)THEN !! ALL ELEMENTS ARE DIFFERENT
	 D(1)=V(1)**2/((V(1)-V(2))*(V(1)-V(3))*(V(1)-V(4)))
	 D(2)=V(2)**2/((V(2)-V(1))*(V(2)-V(3))*(V(2)-V(4)))
	 D(3)=V(3)**2/((V(3)-V(1))*(V(3)-V(2))*(V(3)-V(4)))
	 D(4)=-D(1)-D(2)-D(3)
	 A(1)=0.D0
	  DO 50 I=1,4
	  ALN(I)=LOG(ABS(V(I)))
	  A(1)=A(1)+3.D0*D(I)*ALN(I)
  50      CONTINUE
	 DO 70 J=2,4
	 A(J)=0.D0
	  DO 60 I=1,4
	  IF(I.NE.J)THEN
	  A(J)=A(J)+V(I)*D(I)*(ALN(I)-ALN(J))/(V(I)-V(J))
	  ELSE
	  A(J)=A(J)+D(J)
	  ENDIF     
  60      CONTINUE
  70     CONTINUE
C
	ELSEIF(ICASE.EQ.1)THEN !! TWO ELEMENTS ARE EQUAL
	 DO 80 K=1,6
	 IF(ICON(K).EQ.1)K0=K
  80     CONTINUE
	 VV=V(LL(K0))          !! EQUAL ELEMENTS
	 VV1=V(LL(7-K0))
	 VV2=V(MM(7-K0))
	 ALNV=LOG(ABS(VV))
	 ALNV1=LOG(ABS(VV1))
	 ALNV2=LOG(ABS(VV2))
	 DD1=1.D0/((VV1-VV)*(VV1-VV)*(VV1-VV2))
	 DD2=1.D0/((VV2-VV)*(VV2-VV)*(VV2-VV1))
	 DD3=1.D0/((VV-VV1)*(VV-VV2))
	 A(1)=3.D0*VV1**2*DD1*(ALNV1-ALNV)+3.D0*VV2**2*DD2*(ALNV2-ALNV)+
     &   3.D0*VV*DD3
	 SS1=DD3/(VV-VV1)
	 SS2=DD3/(VV-VV2)
	 OO2=3.D0-VV/(VV-VV1)-2.D0*VV/(VV-VV2)
	 OO3=3.D0-VV/(VV-VV2)-2.D0*VV/(VV-VV1)
	 T(2)=VV**2*SS2*OO2*(ALNV-ALNV2)+VV1**3*DD1/(VV1-VV2)*
     &   (ALNV1-ALNV2)+VV**2*SS2+VV2**2*DD2
	 T(3)=VV**2*SS1*OO3*(ALNV-ALNV1)+VV2**3*DD2/(VV2-VV1)*
     &   (ALNV2-ALNV1)+VV**2*SS1+VV1**2*DD1
	 T(4)=VV1**3*DD1/(VV1-VV)*(ALNV1-ALNV)+VV2**3*DD2/(VV2-VV)*
     &   (ALNV2-ALNV)+2.5D0*VV*DD3-VV**2*(SS1+SS2)
	 DO 90 I=2,4
	 A(I)=T(NN2(I,K0))
  90     CONTINUE
C
	ELSEIF(ICASE.EQ.2)THEN  !! TWO PAIRS ARE EQUAL
	 K=0
	 DO 100 I=1,6
	 IF(ICON(I).EQ.1)THEN
	 K=K+1
	 KK(K)=I
	 ENDIF
 100     CONTINUE
	 VV=V(LL(KK(1)))
	 VV1=V(LL(KK(2)))
C  NEW FORMULAE
	 XX=VV1/VV
	 XX1=VV/VV1
	 ALOGXX=LOG(ABS(XX))
	 A(1)=VV*VV1/(VV-VV1)**3*
     &   (6.D0*ALOGXX+3.D0*(XX1-XX))
	 T(2)=VV*VV1/(VV-VV1)**3*
     &   (3.D0/(1.D0-XX1)*ALOGXX+2.5D0*(XX1-1.D0)-2.D0*XX1-XX)
	 T(3)=VV*VV1/(VV-VV1)**3*
     &   (3.D0/(1.D0-XX)*ALOGXX+2.5D0*(1.D0-XX)+2.D0*XX+XX1)
C  OLD FORMULAE
C	 ALNV=LOG(ABS(VV))
C	 ALNV1=LOG(ABS(VV1))
C	 DD=1.D0/(VV-VV1)**2
C	 DD2=DD*DD
C	 A(1)=6.D0*VV*VV1*DD2*(VV-VV1)*(ALNV1-ALNV)+3.D0*(VV+VV1)*DD
C	 T(2)=3.D0*VV*VV1**2*DD2*(ALNV-ALNV1)+2.5D0*VV*DD+(2.D0*VV**2+
C    &   VV1**2)*DD2*(VV1-VV)
C	 T(3)=3.D0*VV1*VV**2*DD2*(ALNV1-ALNV)+2.5D0*VV1*DD+(2.D0*VV1**2+
C    &   VV**2)*DD2*(VV-VV1)
 	 DO 110 I=2,4
 	 A(I)=T(NN3(I,KK(1)))
 110     CONTINUE
C
	ELSEIF(ICASE.EQ.3)THEN   !! THREE ELEMENTS ARE EQUAL
	 K=0
	 DO 120 I=1,6
	 IF(ICON(I).EQ.1)THEN
	 K=K+I
	 K0=I
	 ENDIF
 120     CONTINUE
	 M0=(K-5)/2
	 VV=V(LL(K0))
	 VV1=V(KP(M0))
C  NEW FORMULAE
	 XX=VV1/VV
	 XX1=VV/VV1
	 ALOGXX=LOG(ABS(XX))
	 A(1)=VV1*VV/(VV1-VV)**3*
     &   (3.D0*XX*ALOGXX+6.D0-4.5D0*XX-1.5D0*XX1)
	 T(2)=VV1*VV/(VV1-VV)**3*
     &   (3.D0*XX/(1.D0-XX)*ALOGXX+2.5D0-0.5D0*XX1+XX)
	 T(3)=VV1*VV/(VV1-VV)**3*
     &   (XX/(1.D0-XX1)*ALOGXX-11.D0/6.D0*XX*(1.D0-XX1)**2+
     &   1.5D0*XX1-2.5D0)
C  OLD FORMULAE
C        DD=1.D0/(VV1-VV)**2
C        DD1=DD/(VV1-VV)
C        DD2=DD*DD
C        ALNV=LOG(ABS(VV))
C        ALNV1=LOG(ABS(VV1))
C        A(1)=3*VV1**2*DD1*(ALNV1-ALNV)+DD1*(6.D0*VV*VV1-
C    &   4.5D0*VV1**2-1.5D0*VV**2)
C        T(2)=3.D0*VV*VV1**2*DD2*(ALNV-ALNV1)+DD*2.5D0*VV+
C    &   DD1*(2.D0*VV**2+VV1**2)
C        T(3)=VV1**3*DD2*(ALNV1-ALNV)+11.D0/(VV-VV1)/6.D0-2.5D0*VV*DD-
C    &   VV**2*DD1
	 DO 130 I=2,4
	 A(I)=T(NN4(I,M0))
 130     CONTINUE
C
	ELSEIF(ICASE.EQ.6)THEN !! ALL ELEMENTS ARE EQUAL
	 A(1)=1.D0/V(1)
	 A(2)=0.25D0*A(1)
	 A(3)=A(2)
	 A(4)=A(3)
	ENDIF
C  NORMALIZATION
    	DO 140 J=1,4
	A(J)=A(J)*VT
 140    CONTINUE
	END


        SUBROUTINE MCTTRINT2(VT,A)
************************************************************
*    The calculation of the integrals over tetrahedron     *
*                                                          *
*        /  3                                              *
*    A=  \ d k  R(k)                                       *
*        /                                                 *
*       ttr                                                *
*                                                          *
*    where R(k) are 1,kx,ky,kz (for the matrix elements    *
*    evaluation). The linear interpolation is supposed.    *
*    Written by S.Yu.Savrasov (Lebedev Physical Institute) *
************************************************************
        IMPLICIT REAL*8 (A-H,O-Z)
        DIMENSION A(4)
        A(1)=VT
        DO 10 I=2,4
        A(I)=VT*0.25D0
  10    CONTINUE
        END

