	SUBROUTINE SCF1
************************************************************
*   The calculation of the energy band structure in terms  *
*   of LMTO method (O.K.Andersen,Phys.Rev.B12,3060(1975))  *
*   Full potential terms after S.Yu.Savrasov and D.Yu.     *
*   Savrasov, Phys.Rev.B46, 12181 (1992).                  *
*   Written by S.Yu.Savrasov (P.N.Lebedev Physical Instit.)*
************************************************************
	IMPLICIT REAL*8 (A-H,O-Z)
	CHARACTER*8 ST,FN
	COMMON /CONTROL/MANAGE                 !!! from init
	COMMON /SCF1COM/NITER1                 !!! from init
	COMMON /SPIN/   NSPIN                  !!! from init
	COMMON /UNIT/   IUN                    !!! from init
	LOGICAL EXIT
	DATA ST/'started'/,FN/'finished'/
C  CALCULATE BARE STRUCTURE CONSTANTS
	call timel(times)
	write(iun,1001)st,times
        CALL STRMSH         !!!
	call timel(timef)
	write(iun,1001)fn,timef
C  CALCULATE CELL CONSTANTS
	call timel(times)
	write(iun,1002)st,times
	CALL SCELLS         !!! atomic cell enevelopes
        CALL SRFMSH         !!! surface special points
        CALL ENVMSH         !!! generate cluster sites
        CALL READHOP        !!! read hopping integrals
	call timel(timef)
	write(iun,1002)fn,timef
C  CALCULATE SCREENED STRUCTURE CONSTANTS
        call timel(times)
        write(iun,1003)st,times
        CALL ALPCON         !!! get screening parameters "alpha"
        CALL ENVADD         !!! get three-center point indicator
        CALL SCRCON         !!! screen bare structure constants
        call timel(timef)
        write(iun,1003)fn,timef
	IF(MANAGE.LE.1)CALL OUTPUT('No band structure calculation')
	CALL READRO(0)      !!! read charge density
	CALL SPLACE(0)      !!! save Dny for admixing
        CALL READSV(0)      !!! read spherical potential if exists
	CALL SMULTIN        !!! finds multipoles in the int.region
	CALL SMIXTCH(0)     !!! save multipoles for mixing
C  *****  THE LOOP OF SELF-CONSISTENCY *****
	DO 100 ITER=1,NITER1
	write(iun,1010)iter
	CALL ROCHEB         !!! find Chebyshev expansion coefficients
	CALL SMULTRO        !!! calculate multipole charges
	CALL PRINTC(1)      !!! save input charge for printing
        CALL MIXROB(ITER,1) !!! initialize Broyden mixing
        CALL MIXHUB(ITER,1) !!! initialize mixing for occupancies
C  CALCULATE FULL POTENTIAL
	call timel(times)
	write(iun,1004)st,times
	CALL VCOUL          !!! calculate Coulomb potential within MT-spheres
	CALL VCINT          !!! calculate Coulomb potential in the int.region
	CALL VEXCH(1)       !!! calculate ex.-cor.potential within MT-spheres
	CALL VXINT(1)       !!! calculate ex.-cor.potential in the int.region
	CALL ZEROV          !!! set up energy zero
	CALL VCHEB          !!! find Chebyshev expansion coefficients
	CALL PRINT2         !!! print necessary information
        CALL HUBPOT         !!! adds Hubbard correction to the potential
        CALL PRINTH(IUN)    !!! print Hubbard correction
	CALL WRITEV(ITER)   !!! store the potential
	call timel(timef)
	write(iun,1004)fn,timef
C  CALCULATE BAND STRUCTURE
	call timel(times)
	write(iun,1005)st,times
        CALL BANDS          !!! find band energies and eigenvectors
	call timel(timef)
	write(iun,1005)fn,timef
	IF(MANAGE.LE.2)CALL OUTPUT('No SCF1 loop')
C  CALCULATE BRILLOUIN ZONE INTEGRALS
	call timel(times)
	write(iun,1006)st,times
        CALL BZINT          !!! find Fermi level and partial numbers of states
	call timel(timef)
	write(iun,1006)fn,timef
C  CALCULATE FULL VALENCE PLUS SEMICORE DENSITY IN UNIT-CELL
	call timel(times)
	write(iun,1007)st,times
	CALL RHOFUL         !!! find new valence charge density
	CALL SRODOT         !!! find change in density due to Eny
	CALL SROINT         !!! find valence density in the int.region
        CALL ORBMAG         !!! find orbital contribution to magnetization
	CALL RENROV         !!! renormalize density
        CALL RHOHUB         !!! set occupation numbers
	call timel(timef)
	write(iun,1007)fn,timef
C  CALCULATE DEEP CORE
	call timel(times)
	write(iun,1008)st,times
	CALL RENCOR(ITER)   !!! find core charge density destribution in cell
	call timel(timef)
	write(iun,1008)fn,timef
	CALL ROCHEB         !!! find Chebyshev expansion coefficients
	CALL SMULTIN        !!! find multipoles in the int.region
	CALL SMULTRO        !!! calculate multipole charges
        CALL PRINTM(IUN)    !!! print occupation numbers
        CALL WRITEHUB       !!! store occupation numbers
C  CALCULATE TOTAL ENERGY AND FORCES
	call timel(times)
	write(iun,1009)st,times
        CALL SENERGY(EXIT)  !!! calculate Etot in LDA
	CALL FORCES         !!! calculate forces
	call timel(timef)
	write(iun,1009)fn,timef
	CALL PRINTC(2)      !!! print input-output charge transfer
	IF(EXIT)CALL OUTPUT('Total energy convergence reached')
C  ADMIX DENSITY FOR THE NEXT ITERATION
        CALL MIXROB(ITER,2) !!! mix densities according to Broyden
	CALL MIXRO1(ITER,EXIT)! mix densities with linear mixing
        CALL MIXHUB(ITER,2) !!! mix occupancies
	CALL SPLACE(ITER)   !!! place Eny at the centers of gravity
	CALL SMIXTCH(ITER)  !!! mix multipole charges
	CALL WRITERO(ITER)  !!! store charge density
	IF(EXIT)CALL OUTPUT('Charge density convergence reached')
 100	CONTINUE
	CALL OUTPUT('Number of iterations exceeded in the SCF1 loop')
1001  FORMAT('******* STRMSH ',A8,' ; CPU time: ',G14.7,' ************')
1002  FORMAT('******* SRFMSH ',A8,' ; CPU time: ',G14.7,' ************')
1003  FORMAT('******* SCRCON ',A8,' ; CPU time: ',G14.7,' ************')
1004  FORMAT('*******  VFULL ',A8,' ; CPU time: ',G14.7,' ************')
1005  FORMAT('******** BANDS ',A8,' ; CPU time: ',G14.7,' ************')
1006  FORMAT('******** BZINT ',A8,' ; CPU time: ',G14.7,' ************')
1007  FORMAT('******* ROFULL ',A8,' ; CPU time: ',G14.7,' ************')
1008  FORMAT('******* RENCOR ',A8,' ; CPU time: ',G14.7,' ************')
1009  FORMAT('******* ENERGY ',A8,' ; CPU time: ',G14.7,' ************')
1010  FORMAT(/'                ITERATION ',I3/)
	END
